@@ -3,7 +3,7 @@ import sys
 import warnings
 
 from neomodel import config
-from neomodel.exceptions import DoesNotExist, ClassAlreadyDefined
+from neomodel.exceptions import DoesNotExist, NodeClassAlreadyDefined
 from neomodel.hooks import hooks
 from neomodel.properties import Property, PropertyManager
 from neomodel.util import Database, classproperty, _UnsavedNode, _get_node_properties
@@ -32,7 +32,7 @@ def drop_constraints(quiet=True, stdout=None):
         else:
             db.cypher_query('DROP CONSTRAINT {!s}'.format(constraint[0]))
             match = pattern.search(constraint[1])
-        stdout.write(''' - Droping unique constraint and index on label {0} with property {1}.\n'''.format(
+        stdout.write(''' - Dropping unique constraint and index on label {0} with property {1}.\n'''.format(
             match.group(1), match.group(2)))
     stdout.write("\n")
 
@@ -74,10 +74,10 @@ def remove_all_labels(stdout=None):
     if not stdout:
         stdout = sys.stdout
 
-    stdout.write("Droping constraints...\n")
+    stdout.write("Dropping constraints...\n")
     drop_constraints(quiet=False, stdout=stdout)
 
-    stdout.write('Droping indexes...\n')
+    stdout.write('Dropping indexes...\n')
     drop_indexes(quiet=False, stdout=stdout)
 
 
@@ -110,7 +110,8 @@ def install_labels(cls, quiet=True, stdout=None):
                 db.cypher_query("CREATE INDEX on :{0}({1}); ".format(
                     cls.__label__, db_property))
             except ClientError as e:
-                if str(e.message).lower().startswith("an equivalent index already exists"):
+                if e.code in ('Neo.ClientError.Schema.EquivalentSchemaRuleAlreadyExists',
+                              'Neo.ClientError.Schema.IndexAlreadyExists'):
                     stdout.write('{0}\n'.format(str(e)))
                 else:
                     raise
@@ -124,7 +125,8 @@ def install_labels(cls, quiet=True, stdout=None):
                                 "on (n:{0}) ASSERT n.{1} IS UNIQUE".format(
                     cls.__label__, db_property))
             except ClientError as e:
-                if str(e.message).lower().startswith("an equivalent constraint already exists"):
+                if e.code in ('Neo.ClientError.Schema.EquivalentSchemaRuleAlreadyExists',
+                              'Neo.ClientError.Schema.ConstraintAlreadyExists'):
                     stdout.write('{0}\n'.format(str(e)))
                 else:
                     raise
@@ -204,7 +206,7 @@ class NodeMeta(type):
             if label_set not in db._NODE_CLASS_REGISTRY:
                 db._NODE_CLASS_REGISTRY[label_set] = cls
             else:
-                raise ClassAlreadyDefined(cls, db._NODE_CLASS_REGISTRY)
+                raise NodeClassAlreadyDefined(cls, db._NODE_CLASS_REGISTRY)
 
         return cls
 
