@@ -1,10 +1,12 @@
 from .core import StructuredNode, db
 from .properties import AliasProperty
-from .exception import MultipleNodesReturned
+from .exceptions import MultipleNodesReturned
+from .match_q import Q, QBase
 import inspect
 import re
 OUTGOING, INCOMING, EITHER = 1, -1, 0
 
+
 # basestring python 3.x fallback
 try:
     basestring
@@ -44,7 +46,7 @@ def _rel_helper(lhs, rhs, ident=None, relation_type=None, direction=None, relati
 
     if relation_properties:
         rel_props = ' {{{0}}}'.format(', '.join(
-            ['{}: {}'.format(key, value) for key, value in relation_properties.items()]))
+            ['{0}: {1}'.format(key, value) for key, value in relation_properties.items()]))
 
     # direct, relation_type=None is unspecified, relation_type
     if relation_type is None:
@@ -54,11 +56,63 @@ def _rel_helper(lhs, rhs, ident=None, relation_type=None, direction=None, relati
         stmt = stmt.format('[*]')
     else:
         # explicit relation_type
-        stmt = stmt.format('[%s:`%s`%s]' % (ident if ident else '', relation_type, rel_props))
+        stmt = stmt.format('[{0}:`{1}`{2}]'.format(ident if ident else '', relation_type, rel_props))
 
     return "({0}){1}({2})".format(lhs, stmt, rhs)
 
 
+def _rel_merge_helper(lhs, rhs, ident='neomodelident', relation_type=None, direction=None, relation_properties=None, **kwargs):
+    """
+    Generate a relationship merging string, with specified parameters.
+    Examples:
+    relation_direction = OUTGOING: (lhs)-[relation_ident:relation_type]->(rhs)
+    relation_direction = INCOMING: (lhs)<-[relation_ident:relation_type]-(rhs)
+    relation_direction = EITHER: (lhs)-[relation_ident:relation_type]-(rhs)
+
+    :param lhs: The left hand statement.
+    :type lhs: str
+    :param rhs: The right hand statement.
+    :type rhs: str
+    :param ident: A specific identity to name the relationship, or None.
+    :type ident: str
+    :param relation_type: None for all direct rels, * for all of any length, or a name of an explicit rel.
+    :type relation_type: str
+    :param direction: None or EITHER for all OUTGOING,INCOMING,EITHER. Otherwise OUTGOING or INCOMING.
+    :param relation_properties: dictionary of relationship properties to merge
+    :returns: string
+    """
+
+    if direction == OUTGOING:
+        stmt = '-{0}->'
+    elif direction == INCOMING:
+        stmt = '<-{0}-'
+    else:
+        stmt = '-{0}-'
+
+    rel_props = ''
+    rel_none_props = ''
+
+    if relation_properties:
+        rel_props = ' {{{0}}}'.format(', '.join(
+            ['{0}: {1}'.format(key, value) for key, value in relation_properties.items() if value is not None]))
+        if None in relation_properties.values():
+            rel_none_props = ' ON CREATE SET {0} ON MATCH SET {0}'.format(
+                ', '.join(
+                    ['{0}.{1}={2}'.format(ident, key, '${!s}'.format(key)) for key, value in relation_properties.items() if value is None])
+            )
+    # direct, relation_type=None is unspecified, relation_type
+    if relation_type is None:
+        stmt = stmt.format('')
+    # all("*" wildcard) relation_type
+    elif relation_type == '*':
+        stmt = stmt.format('[*]')
+    else:
+        # explicit relation_type
+        stmt = stmt.format('[{0}:`{1}`{2}]'.format(ident, relation_type, rel_props))
+
+    return "({0}){1}({2}){3}".format(lhs, stmt, rhs, rel_none_props)
+
+
 # special operators
 _SPECIAL_OPERATOR_IN = 'IN'
 _SPECIAL_OPERATOR_INSENSITIVE = '(?i)'
@@ -115,12 +169,12 @@ def install_traversals(cls, node_set):
 
     for key, value in rels.items():
         if hasattr(node_set, key):
-            raise ValueError("Can't install traversal '{}' exists on NodeSet".format(key))
+            raise ValueError("Can't install traversal '{0}' exists on NodeSet".format(key))
 
         rel = getattr(cls, key)
         rel._lookup_node_class()
 
-        traversal = Traversal(source=node_set, key=key, definition=rel.definition)
+        traversal = Traversal(source=node_set, name=key, definition=rel.definition)
         setattr(node_set, key, traversal)
 
 
@@ -134,14 +188,14 @@ def process_filter_args(cls, kwargs):
 
     for key, value in kwargs.items():
         if '__' in key:
-            prop, operator = key.split('__')
+            prop, operator = key.rsplit('__')
             operator = OPERATOR_TABLE[operator]
         else:
             prop = key
             operator = '='
 
         if prop not in cls.defined_properties(rels=False):
-            raise ValueError("No such property {} on {}".format(prop, cls.__name__))
+            raise ValueError("No such property {0} on {1}".format(prop, cls.__name__))
 
         property_obj = getattr(cls, prop)
         if isinstance(property_obj, AliasProperty):
@@ -151,17 +205,17 @@ def process_filter_args(cls, kwargs):
             # handle special operators
             if operator == _SPECIAL_OPERATOR_IN:
                 if not isinstance(value, tuple) and not isinstance(value, list):
-                    raise ValueError('Value must be a tuple or list for IN operation {}={}'.format(key, value))
+                    raise ValueError('Value must be a tuple or list for IN operation {0}={1}'.format(key, value))
                 deflated_value = [property_obj.deflate(v) for v in value]
             elif operator == _SPECIAL_OPERATOR_ISNULL:
                 if not isinstance(value, bool):
-                    raise ValueError('Value must be a bool for isnull operation on {}'.format(key))
+                    raise ValueError('Value must be a bool for isnull operation on {0}'.format(key))
                 operator = 'IS NULL' if value else 'IS NOT NULL'
                 deflated_value = None
             elif operator in _REGEX_OPERATOR_TABLE.values():
                 deflated_value = property_obj.deflate(value)
                 if not isinstance(deflated_value, basestring):
-                    raise ValueError('Must be a string value for {}'.format(key))
+                    raise ValueError('Must be a string value for {0}'.format(key))
                 if operator in _STRING_REGEX_OPERATOR_TABLE.values():
                     deflated_value = re.escape(deflated_value)
                 deflated_value = operator.format(deflated_value)
@@ -187,7 +241,7 @@ def process_has_args(cls, kwargs):
 
     for key, value in kwargs.items():
         if key not in rel_definitions:
-            raise ValueError("No such relation {} defined on a {}".format(key, cls.__name__))
+            raise ValueError("No such relation {0} defined on a {1}".format(key, cls.__name__))
 
         rhs_ident = key
 
@@ -237,8 +291,8 @@ class QueryBuilder(object):
             if hasattr(source, '_order_by'):
                 self.build_order_by(ident, source)
 
-            if source.filters:
-                self.build_where_stmt(ident, source.filters)
+            if source.filters or source.q_filters:
+                self.build_where_stmt(ident, source.filters, source.q_filters, source_class=source.source_class)
 
             return ident
         elif isinstance(source, StructuredNode):
@@ -252,10 +306,10 @@ class QueryBuilder(object):
 
     def build_order_by(self, ident, source):
         if '?' in source._order_by:
-            self._ast['with'] = '{}, rand() as r'.format(ident)
+            self._ast['with'] = '{0}, rand() as r'.format(ident)
             self._ast['order_by'] = 'r'
         else:
-            self._ast['order_by'] = ['{}.{}'.format(ident, p)
+            self._ast['order_by'] = ['{0}.{1}'.format(ident, p)
                                      for p in source._order_by]
 
     def build_traversal(self, traversal):
@@ -282,11 +336,12 @@ class QueryBuilder(object):
 
     def build_node(self, node):
         ident = node.__class__.__name__.lower()
-        if 'start' not in self._ast:
-            self._ast['start'] = []
-
         place_holder = self._register_place_holder(ident)
-        self._ast['start'].append('{} = node({{{}}})'.format(ident, place_holder))
+
+        # Hack to emulate START to lookup a node by id
+        _node_lookup = 'MATCH ({0}) WHERE id({1})=${2} WITH {3}'.format(ident, ident, place_holder, ident)
+        self._ast['lookup'] = _node_lookup
+
         self._query_params[place_holder] = node.id
 
         self._ast['return'] = ident
@@ -298,7 +353,7 @@ class QueryBuilder(object):
         match nodes by a label
         """
         ident_w_label = ident + ':' + cls.__label__
-        self._ast['match'].append('({})'.format(ident_w_label))
+        self._ast['match'].append('({0})'.format(ident_w_label))
         self._ast['return'] = ident
         self._ast['result_class'] = cls
         return ident
@@ -332,41 +387,71 @@ class QueryBuilder(object):
             self._place_holder_registry[key] = 1
         return key + '_' + str(self._place_holder_registry[key])
 
-    def build_where_stmt(self, ident, filters):
+    def _parse_q_filters(self, ident, q, source_class):
+        target = []
+        for child in q.children:
+            if isinstance(child, QBase):
+                q_childs = self._parse_q_filters(ident, child, source_class)
+                if child.connector == Q.OR:
+                    q_childs = "(" + q_childs + ")"
+                target.append(q_childs)
+            else:
+                kwargs = {child[0]: child[1]}
+                filters = process_filter_args(source_class, kwargs)
+                for prop, op_and_val in filters.items():
+                    op, val = op_and_val
+                    if op in _UNARY_OPERATORS:
+                        # unary operators do not have a parameter
+                        statement = '{0}.{1} {2}'.format(ident, prop, op)
+                    else:
+                        place_holder = self._register_place_holder(ident + '_' + prop)
+                        statement = '{0}.{1} {2} ${3}'.format(ident, prop, op, place_holder)
+                        self._query_params[place_holder] = val
+                    target.append(statement)
+        ret = ' {0} '.format(q.connector).join(target)
+        if q.negated:
+            ret = 'NOT ({0})'.format(ret)
+        return ret
+
+    def build_where_stmt(self, ident, filters, q_filters=None, source_class=None):
         """
         construct a where statement from some filters
         """
-        stmts = []
-        for row in filters:
-            negate = False
+        if q_filters is not None:
+            stmts = self._parse_q_filters(ident, q_filters, source_class)
+            if stmts:
+                self._ast['where'].append(stmts)
+        else:
+            stmts = []
+            for row in filters:
+                negate = False
 
-            # pre-process NOT cases as they are nested dicts
-            if '__NOT__' in row and len(row) == 1:
-                negate = True
-                row = row['__NOT__']
+                # pre-process NOT cases as they are nested dicts
+                if '__NOT__' in row and len(row) == 1:
+                    negate = True
+                    row = row['__NOT__']
 
-            for prop, op_and_val in row.items():
-                op, val = op_and_val
-                if op in _UNARY_OPERATORS:
-                    # unary operators do not have a parameter
-                    statement = '{} {}.{} {}'.format('NOT' if negate else '', ident, prop, op)
-                else:
-                    place_holder = self._register_place_holder(ident + '_' + prop)
-                    statement = '{} {}.{} {} {{{}}}'.format('NOT' if negate else '', ident, prop, op, place_holder)
-                    self._query_params[place_holder] = val
-                stmts.append(statement)
+                for prop, op_and_val in row.items():
+                    op, val = op_and_val
+                    if op in _UNARY_OPERATORS:
+                        # unary operators do not have a parameter
+                        statement = '{0} {1}.{2} {3}'.format('NOT' if negate else '', ident, prop, op)
+                    else:
+                        place_holder = self._register_place_holder(ident + '_' + prop)
+                        statement = '{0} {1}.{2} {3} ${4}'.format('NOT' if negate else '', ident, prop, op, place_holder)
+                        self._query_params[place_holder] = val
+                    stmts.append(statement)
 
-        self._ast['where'].append(' AND '.join(stmts))
+            self._ast['where'].append(' AND '.join(stmts))
 
     def build_query(self):
         query = ''
 
-        if 'start' in self._ast:
-            query += 'START '
-            query += ', '.join(self._ast['start'])
+        if 'lookup' in self._ast:
+            query += self._ast['lookup']
 
         query += ' MATCH '
-        query += ', '.join(['({})'.format(i) for i in self._ast['match']])
+        query += ', '.join(['({0})'.format(i) for i in self._ast['match']])
 
         if 'where' in self._ast and self._ast['where']:
             query += ' WHERE '
@@ -391,7 +476,7 @@ class QueryBuilder(object):
         return query
 
     def _count(self):
-        self._ast['return'] = 'count({})'.format(self._ast['return'])
+        self._ast['return'] = 'count({0})'.format(self._ast['return'])
         # drop order_by, results in an invalid query
         self._ast.pop('order_by', None)
         query = self.build_query()
@@ -402,18 +487,25 @@ class QueryBuilder(object):
         # inject id = into ast
         ident = self._ast['return']
         place_holder = self._register_place_holder(ident + '_contains')
-        self._ast['where'].append('id({}) = {{{}}}'.format(ident, place_holder))
+        self._ast['where'].append('id({0}) = ${1}'.format(ident, place_holder))
         self._query_params[place_holder] = node_id
-        return self._count() == 1
+        return self._count() >= 1
 
-    def _execute(self):
+    def _execute(self, lazy=False):
+        if lazy:
+            # inject id = into ast
+            self._ast['return'] = 'id({})'.format(self._ast['return'])
         query = self.build_query()
-        results, _ = db.cypher_query(query, self._query_params)
+        results, _ = db.cypher_query(query, self._query_params, resolve_objects=True)            
+        # The following is not as elegant as it could be but had to be copied from the 
+        # version prior to cypher_query with the resolve_objects capability.
+        # It seems that certain calls are only supposed to be focusing to the first 
+        # result item returned (?)
         if results:
-            return [self._ast['result_class'].inflate(n[0]) for n in results]
+            return [n[0] for n in results]
         return []
-
-
+        
+        
 class BaseSet(object):
     """
     Base class for all node sets.
@@ -422,13 +514,14 @@ class BaseSet(object):
     """
     query_cls = QueryBuilder
 
-    def all(self):
+    def all(self, lazy=False):
         """
         Return all nodes belonging to the set
+        :param lazy: False by default, specify True to get nodes with id only without the parameters.
         :return: list of nodes
         :rtype: list
         """
-        return self.query_cls(self).build_ast()._execute()
+        return self.query_cls(self).build_ast()._execute(lazy)
 
     def __iter__(self):
         return (i for i in self.query_cls(self).build_ast()._execute())
@@ -488,23 +581,26 @@ class NodeSet(BaseSet):
         install_traversals(self.source_class, self)
 
         self.filters = []
+        self.q_filters = Q()
 
         # used by has()
         self.must_match = {}
         self.dont_match = {}
 
-    def get(self, **kwargs):
+    def _get(self, limit=None, lazy=False, **kwargs):
+        self.filter(**kwargs)
+        if limit:
+            self.limit = limit
+        return self.query_cls(self).build_ast()._execute(lazy)
+
+    def get(self, lazy=False, **kwargs):
         """
         Retrieve one node from the set matching supplied parameters
-
+        :param lazy: False by default, specify True to get nodes with id only without the parameters.
         :param kwargs: same syntax as `filter()`
         :return: node
         """
-        output = process_filter_args(self.source_class, kwargs)
-        if output:
-            self.filters.append(output)
-        self.limit = 2
-        result = self.query_cls(self).build_ast()._execute()
+        result = self._get(limit=2, lazy=lazy, **kwargs)
         if len(result) > 1:
             raise MultipleNodesReturned(repr(kwargs))
         elif not result:
@@ -524,7 +620,32 @@ class NodeSet(BaseSet):
         except self.source_class.DoesNotExist:
             pass
 
-    def filter(self, **kwargs):
+    def first(self, **kwargs):
+        """
+        Retrieve the first node from the set matching supplied parameters
+
+        :param kwargs: same syntax as `filter()`
+        :return: node
+        """
+        result = result = self._get(limit=1, **kwargs)
+        if result:
+            return result[0]
+        else:
+            raise self.source_class.DoesNotExist(repr(kwargs))
+
+    def first_or_none(self, **kwargs):
+        """
+        Retrieve the first node from the set matching supplied parameters or return none
+
+        :param kwargs: same syntax as `filter()`
+        :return: node or none
+        """
+        try:
+            return self.first(**kwargs)
+        except self.source_class.DoesNotExist:
+            pass
+
+    def filter(self, *args, **kwargs):
         """
         Apply filters to the existing nodes in the set.
 
@@ -555,22 +676,19 @@ class NodeSet(BaseSet):
 
         :return: self
         """
-
-        output = process_filter_args(self.source_class, kwargs)
-        if output:
-            self.filters.append(output)
+        if args or kwargs:
+            self.q_filters = Q(self.q_filters & Q(*args, **kwargs))
         return self
 
-    def exclude(self, **kwargs):
+    def exclude(self, *args, **kwargs):
         """
         Exclude nodes from the NodeSet via filters.
 
         :param kwargs: filter parameters see syntax for the filter method
         :return: self
         """
-        output = process_filter_args(self.source_class, kwargs)
-        if output:
-            self.filters.append({'__NOT__': output})
+        if args or kwargs:
+            self.q_filters = Q(self.q_filters & ~Q(*args, **kwargs))
         return self
 
     def has(self, **kwargs):
@@ -601,7 +719,7 @@ class NodeSet(BaseSet):
                     desc = False
 
                 if prop not in self.source_class.defined_properties(rels=False):
-                    raise ValueError("No such property {} on {}".format(
+                    raise ValueError("No such property {0} on {1}".format(
                         prop, self.source_class.__name__))
 
                 property_obj = getattr(self.source_class, prop)
@@ -615,15 +733,23 @@ class NodeSet(BaseSet):
 
 class Traversal(BaseSet):
     """
-    Models a traversal from a node to another, inherits from BaseSet
+    Models a traversal from a node to another.
+
+    :param source: Starting of the traversal.
+    :type source: A :class:`~neomodel.core.StructuredNode` subclass, an
+                  instance of such, a :class:`~neomodel.match.NodeSet` instance
+                  or a :class:`~neomodel.match.Traversal` instance.
+    :param name: A name for the traversal.
+    :type name: :class:`str`
+    :param definition: A relationship definition that most certainly deserves
+                       a documentation here.
+    :type defintion: :class:`dict`
     """
 
-    def __init__(self, source, key, definition):
+    def __init__(self, source, name, definition):
         """
         Create a traversal
-        :param source: start of traversal could be any of: StructuredNode instance, StucturedNode class, NodeSet
-        :param key:
-        :param definition: relationship definition
+
         """
         self.source = source
 
@@ -636,11 +762,21 @@ class Traversal(BaseSet):
         elif isinstance(source, NodeSet):
             self.source_class = source.source_class
         else:
-            raise ValueError("Bad source for traversal: {}".format(repr(source)))
+            raise TypeError("Bad source for traversal: "
+                            "{0}".format(type(source)))
+
+        invalid_keys = (
+                set(definition) - {'direction', 'model', 'node_class', 'relation_type'}
+        )
+        if invalid_keys:
+            raise ValueError(
+                'Unallowed keys in Traversal definition: {invalid_keys}'
+                .format(invalid_keys=invalid_keys)
+            )
 
         self.definition = definition
         self.target_class = definition['node_class']
-        self.name = key
+        self.name = name
         self.filters = []
 
     def match(self, **kwargs):
@@ -652,10 +788,10 @@ class Traversal(BaseSet):
         :param kwargs: see `NodeSet.filter()` for syntax
         :return: self
         """
-        if 'model' not in self.definition:
-            raise ValueError("match() only available on relationships with a model")
         if kwargs:
-            output = process_filter_args(self.definition['model'], kwargs) 
+            if self.definition.get('model') is None:
+                raise ValueError("match() with filter only available on relationships with a model")
+            output = process_filter_args(self.definition['model'], kwargs)
             if output:
                 self.filters.append(output)
         return self
